/*
 * ethernetThreadPool.c -- contains functions that allows to:
 * - create threads responsible to listen coming frames in each ethernet port and make decision down the port that corresponds to a loop.
 * - create thread that send periodically loop discovery frame and make decision to up again an ethernet port
 *
 * Copyright (C) 2017 Inteno Broadband Technology AB. All rights reserved.
 *
 * Author: Omar Kallel <omar.kallel@pivasoftware.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include<stdlib.h>
#include <pcap.h>
#include <pthread.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include "ethernetThreadPool.h"

pthread_mutex_t mutex_send_discovery_ports= PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cv;
uint8_t counter = 0;
uint8_t senddiscoverycheck = 0;
uint8_t sent_after_loop_counter = 0;
uint8_t numberDetectedLoops = 0;

char **portsLoop;

/*
 * functions that allows to control ethernet ports that corresponds to L2 loops
 */
void init_portsLoop(void){
	uint8_t i;
	portsLoop =(char**)malloc(portsnumber*sizeof(char*));
	for(i=0; i<portsnumber; i++) portsLoop[i] = NULL;
}

int portExistInPortsLoop(char *port){
	int i;
	for(i=0; i<numberDetectedLoops; i++) {
		if(!strcmp(portsLoop[i], port)) return 1;
	}
	return 0;
}

void addPortsLoop(char *port){
	if(port != NULL && strlen(port)>0 && !portExistInPortsLoop(port)){
		portsLoop[numberDetectedLoops] = (char*)calloc(strlen(port), sizeof(char));
		sprintf(portsLoop[numberDetectedLoops], "%s", port);
		numberDetectedLoops++;
	}
}

void reactivateAllPortsInPortsLoop(void){
	uint8_t i;
	for(i=0; i<numberDetectedLoops; i++){
		ethernetDeviceUpDown(portsLoop[i], "up");
	}
}

void reinitiatportsloopelements(void){
	uint8_t i;
	for(i=0; i<numberDetectedLoops; i++){
		free(portsLoop[i]);
	}
	numberDetectedLoops = 0;
}

/*
 *Send L2 loop discovery frame periodically
 */

void *sendBroadcastDiscoveryFrame(void *arg){
	pthread_mutex_lock(&mutex_send_discovery_ports);
	pthread_cond_wait(&cv, &mutex_send_discovery_ports);
	pthread_mutex_unlock (&mutex_send_discovery_ports);
	pcap_t **fp_list;
    char error[PCAP_ERRBUF_SIZE];
    uint8_t i, j;

    fp_list =(pcap_t **)malloc(portsnumber * sizeof(pcap_t *));
	for(j=0; j<portsnumber; j++){
	    if((fp_list[j] = pcap_open_live(portsNames[j], 100, 1, 1000, error) ) == NULL)
	    {
	        fprintf(stderr,"\nError opening adapter: %s\n", error);
	        return;
	    }
	}
	while(1){
		for(j=0; j<portsnumber; j++) pcap_sendpacket(fp_list[j], l2loop_discovery_frame, 100);
		senddiscoverycheck =1;
		if(sent_after_loop_counter>0) sent_after_loop_counter++;
		if(sent_after_loop_counter -1 == send_discovery_numbertimes){
			reactivateAllPortsInPortsLoop();
			reinitiatportsloopelements();
			sent_after_loop_counter = 0;
		}
		sleep(send_loop_discovery_period);
	}
}

void init_l2loop_send_discovery_thread(void){
	int iret;
    iret = pthread_create( &l2loop_send_discovery, NULL, sendBroadcastDiscoveryFrame, (void*) "");
	if(iret)
	{
	 fprintf(stderr,"Error - creating thread send discovery return code: %d\n",iret);
	 exit(EXIT_FAILURE);
	}
}

/*
 * Listen to physical ethernet ports
 */

void ethernetCaptureLoopHandler( u_char *args, const struct pcap_pkthdr *packet_header, const u_char *packet_body)
{
    struct ether_header *eptr;  /* net/ethernet.h */
    int test = 1, i;

    /* lets start with the ether header... */
    eptr = (struct ether_header *) packet_body;
    struct ether_addr* ownmac = ether_aton(ownMacAddress);

    for(i=0; i<6; i++){
    	if(eptr->ether_shost[i]!= ownmac->ether_addr_octet[i]){
    		test = 0;
    		break;
    	}
    }

    if(test && senddiscoverycheck==1 && numberDetectedLoops == 0) {
    	senddiscoverycheck = 0;
	fprintf(stdout,"L2 loop is detected on port: %s\n", args);
    	ethernetDeviceUpDown(args, "down");
    	sent_after_loop_counter = 1;
    	addPortsLoop(args);
    }
    return;
}

void *listenEthernetFrame(void *ehternetInterface){
	struct ether_header *eptr;
    char error_buffer[PCAP_ERRBUF_SIZE];
    pcap_t *handle;
    int timeout_limit = 10000; /* In milliseconds */
    char *ethernet = (char *)ehternetInterface;


    counter++;

    /* Open device for live capture */
    handle = pcap_open_live(
    		ethernet,
            BUFSIZ,
            0,
            timeout_limit,
            error_buffer
        );
    if (handle == NULL) {
         fprintf(stderr, "Could not open device %s: %s\n", ethernet, error_buffer);
         return 2;
     }
    if(counter==portsnumber) pthread_cond_signal(&cv);
    pcap_setdirection(handle, PCAP_D_IN);
    pcap_loop(handle, 0, ethernetCaptureLoopHandler, ethernet);
}

void init_ethernet_thread_pool(void)
{
	int iret;
	uint8_t i;
	pthread_attr_t attr;

    size_t stacksize = 14*1024;
    pthread_attr_init(&attr);
    pthread_attr_setstacksize(&attr, stacksize);

    init_portsLoop();
	portsThreads = (pthread_t*)malloc(portsnumber * sizeof(pthread_t));

	for(i=0; i<portsnumber; i++){
	    iret = pthread_create( &portsThreads[i], &attr, listenEthernetFrame, (void*) portsNames[i]);
		if(iret)
		{
		 fprintf(stderr,"Error - creating thread send discovery return code: %d\n",iret);
		 exit(EXIT_FAILURE);
		}
	}
}

/*
 * Join threads
 */

void join_ports_threads(void){
	uint8_t i;
	for(i =0; i<portsnumber; i++) pthread_join(portsThreads[i], NULL);
}
