/*
 * mac_ethernet.c -- contains functions that allows to:
 * - interact with ehternet ports (up and down)
 * - get the equipment mac address
 * - create loop discovery frame that will be sent periodically
 *
 * Copyright (C) 2017 Inteno Broadband Technology AB. All rights reserved.
 *
 * Author: Omar Kallel <omar.kallel@pivasoftware.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <linux/if.h>
#include <netinet/in.h>
#include <uci.h>

char *ownMacAddress = NULL;
uint8_t l2loop_discovery_frame[100] = {0};

typedef struct eth_status {
	int sts;
}ethstatus;

/*
 * activate or desactivate the ethernet port
 */
int ethernetDeviceUpDown(char *interfacename, char* updown){
    struct ifreq ifr;
    int skfd, rc;
    ethstatus status;
    int err = -1;

    strcpy(ifr.ifr_name, interfacename);
    if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        fprintf(stderr, "socket open error\n");
        return -1;
    }

    if (ioctl(skfd, SIOCGIFINDEX, &ifr) < 0 ) {
        fprintf(stderr, "ioctl failed. check if %s exists\n", ifr.ifr_name);
        close(skfd);
        return -1;
    }

    if (strcmp(updown, "up") == 0) {
    	status.sts = 5;
    } else if (strcmp(updown, "down") == 0) {
    	status.sts = 6;
    }

    ifr.ifr_data = (void *)&status;
    err = ioctl(skfd, 35323, &ifr);

    if (err) {
        fprintf(stderr, "not able to act in the eth!\n");
        return err;
    }

    return err;
}

/*
 * get the equipment Mac address
 */
int findOwnMacAddress(void){
	ownMacAddress = (char*)calloc(18, sizeof(char));

    struct ifreq ifr;
    struct ifconf ifc;
    char buf[1024];
    int success = 0;

    ownMacAddress = (char*)calloc(18, sizeof(char));

    int sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
    if (sock == -1) {
    	success = 0;
    	goto test_success;
    }

    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    if (ioctl(sock, SIOCGIFCONF, &ifc) == -1) {
    	success = 0;
    	goto test_success;
    }

    struct ifreq* it = ifc.ifc_req;
    const struct ifreq* const end = it + (ifc.ifc_len / sizeof(struct ifreq));

    for (; it != end; ++it) {
        strcpy(ifr.ifr_name, it->ifr_name);
        if (ioctl(sock, SIOCGIFFLAGS, &ifr) == 0) {
            if (! (ifr.ifr_flags & IFF_LOOPBACK)) { // don't count loopback
                if (ioctl(sock, SIOCGIFHWADDR, &ifr) == 0) {
                    success = 1;
                    break;
                }
            }
        }
        else {
        	success = 0;
        	goto test_success;
        }

    }

    test_success:
    if (success) {
    	unsigned char mac_address[6];
    	memcpy(mac_address, ifr.ifr_hwaddr.sa_data, 6);
        sprintf(ownMacAddress, "%02x:%02x:%02x:%02x:%02x:%02x", mac_address[0], mac_address[1], mac_address[2], mac_address[3], mac_address[4], mac_address[5]);
        return 1;
    }
    else {
    	FILE *nvrammac=NULL;
    	char macreadfile[18]={0};
    	char str[13];
    	int i, k=0;
    	if ((nvrammac = fopen("/proc/nvram/BaseMacAddr", "r")) == NULL) return 0;
    	fscanf(nvrammac,"%[^\n]", macreadfile);
    	macreadfile[17]='\0';
		sscanf(macreadfile,"%2c %2c %2c %2c %2c %2c", str, str+2, str+4, str+6, str+8, str+10);
		str[12]='\0';
		printf("str = %s\n",str);
		for(i =0; i<17; i++){
			if(i%3 == 2) ownMacAddress[i] = ':';
			else{
				ownMacAddress[i] = str[k];
				k++;
			}
		}
		ownMacAddress[17]='\0';
    	return 1;
    }

}

/*
 * create loop discovery frame
 */
void buildL2LoopDiscoveryFrame(void){
	uint8_t i, k;
	size_t len = strlen(ownMacAddress);
    char *macblock=NULL;
    for(k=0; k<6; k++) l2loop_discovery_frame[k] = 255;
    for (i=0; i<len; i++) {
        if(i%3 == 0) {
                asprintf(&macblock, "%c%c%c", ownMacAddress[i], ownMacAddress[i+1], '\0');
                l2loop_discovery_frame[k] = (uint8_t)strtol(macblock, NULL, 16);
                k++;
                free(macblock);
        }
    }

    for(k=12;k<100;k++){
    	l2loop_discovery_frame[k]=k;
    }
}
